Package: cdlsim
Title: Simulation of Cropland Data Layer Products from the USDA
Version: 0.1.0
Authors@R: 
    person("Haley", "Burger", , "A02425259@usu.edu", role = c("aut", "cre", "cph"))
Description: Provides tools to simulate categorical raster data, including
    objects from the 'terra' package. Functions focus on generating edge pixel
    values from reclassified raster data derived from the United States
    Department of Agriculture (USDA) Cropland Data Layer products.
License: MIT + file LICENSE
Depends: R (>= 4.1.0)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, terra, landscapemetrics, readxl, utils, Rcpp
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
LinkingTo: Rcpp
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-04-13 15:02:23 UTC; haleyburger
Author: Haley Burger [aut, cre, cph]
Maintainer: Haley Burger <A02425259@usu.edu>
Repository: CRAN
Date/Publication: 2026-04-20 13:00:21 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 12:04:08 UTC; unix
Archs: cdlsim.so.dSYM

# Generated by roxygen2: do not edit by hand

export(download_cdl_mat_files)
export(get_mat_data)
export(get_trans_mat)
export(simulate_raster_patch)
import(dplyr)
import(landscapemetrics)
import(readxl)
importFrom(Rcpp,sourceCpp)
importFrom(dplyr,intersect)
importFrom(dplyr,union)
importFrom(stats,na.omit)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,unzip)
useDynLib(cdlsim, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cdlsim

<!-- badges: start -->

[![R-CMD-check](https://github.com/burgerhaley97/cdlsim/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/burgerhaley97/cdlsim/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of cdlsim is to simulate USDA CDL data to quantify the
sensitivity of metrics calculated on it. The R package landscapemetrics
is used to implement popular FRAGSTATS style metrics in R.

## Installation

The stable release of **cdlsim** will be available from
[CRAN](https://CRAN.R-project.org) soon:

``` r
install.packages("cdlsim")
```

Until then, you can install the development version of cdlsim from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("burgerhaley97/cdlsim")
```

## Example Usage

This is a basic example which shows how a simple landscape would behave
in our simulation:

library(cdlsim)

# Create a small raster
r <- rast(nrows = 20, ncols = 20)
values(r) <- rep(1:3, length.out = ncell(r))

# Simple transition matrix
trans_mat <- matrix(1/3, nrow = 3, ncol = 3)
rownames(trans_mat) <- colnames(trans_mat) <- c("1", "2", "3")

# Simulation example
simulate_raster_patch(r, trans_mat, iterations = 1)

<img src="man/figures/README-unnamed-chunk-2-1.png" width="100%" />

# make the data frame for bl example
bl_mat_data <- get_mat_data(c("UT"))


# List of categories with their corresponding vectors
# Define the values that represent our classes of interest
non_ag <- c(61:65, 81:83, 87:88, 92, 111:112, 121:124, 131, 141:143, 152, 176, 181, 190, 195)
alfalfa <- c(36:37)
major_ag <- c(1, 2, 5, 12, 13, 22:24, 26, 225:226, 228, 234, 236, 238:241, 254)
all_numbers <- 1:256
ag <- setdiff(all_numbers, c(non_ag, alfalfa, major_ag))

cat_5 <- list(non_ag = non_ag, ag = ag, alfalfa = alfalfa, major_ag = major_ag)

# get the confusion matrix for just 2008
trans_mat_5 <- get_trans_mat(bl_mat_data, cat_5)
trans_mat_08 <- trans_mat_5[[1]]

saveRDS(trans_mat_08, file = "inst/extdata/trans_mat_08.rds")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(cdlsim)

## -----------------------------------------------------------------------------
# Define raster dimensions and classes
ncol <- 90
nrow <- 90
classes <- 3

# Initialize a SpatRaster with the desired dimensions
r_small <- terra::rast(ncol = ncol, nrow = nrow, vals = NA)

# Assign values in 30x30 blocks for each class
r_small[1:30, 1:30] <- 1  
r_small[1:30, 31:60] <- 2  
r_small[1:30, 61:90] <- 3 

r_small[31:60, 1:30] <- 2  
r_small[31:60, 31:60] <- 3  
r_small[31:60, 61:90] <- 1 

r_small[61:90, 1:30] <- 3  
r_small[61:90, 31:60] <- 1  
r_small[61:90, 61:90] <- 2  


## ----fig.width=7, fig.height=5, fig.align="center"----------------------------
# Show original landscape before simulation
terra::plot(r_small, axes = FALSE, main = "Simple Landscape")


## -----------------------------------------------------------------------------
# Define confusion matrix, background not transitioning
conf_mat <- matrix(1/3, nrow = 3, ncol = 3)

# Assign row and column names
rownames(conf_mat) <- c("1", "2", "3")
colnames(conf_mat) <- c("1", "2", "3")

# Print the transition matrix 
print(conf_mat)


## -----------------------------------------------------------------------------
# Set a random seed for reproducibility 
set.seed(1891)

# Simulate the example landscape 5 times
simple_sim <- simulate_raster_patch(original_raster = r_small,
                                       transition_matrix = conf_mat,
                                       iterations = 5)

## ----fig.width=7, fig.height=5, fig.align="center"----------------------------
# Plot the results
terra::plot(simple_sim, axes = FALSE)

## -----------------------------------------------------------------------------
# Load the raster
bl_raster <- terra::rast(system.file("extdata/cropped_raster.tif", package="cdlsim"))

## ----fig.width=7, fig.height=5, fig.align="center"----------------------------
terra::plot(bl_raster, axes = FALSE, main = "Bear Lake Watershed")

## -----------------------------------------------------------------------------
# Define the values that represent our classes of interest
non_ag <- c(61:65, 81:83, 87:88, 92, 111:112, 121:124, 131, 141:143, 152, 176, 181, 190, 195)
alfalfa <- c(36:37)
major_ag <- c(1, 2, 5, 12, 13, 22:24, 26, 225:226, 228, 234, 236, 238:241, 254)
all_numbers <- 1:256
ag <- setdiff(all_numbers, c(non_ag, alfalfa, major_ag))

# reclassify into background = 0, major-ag = 1, alfalfa = 2, non-ag = 3, ag = 4
cdl_reclass <- terra::app(bl_raster, fun = function(x) {
  ifelse(is.na(x), 0,
         ifelse(x %in% major_ag, 1,
                ifelse(x %in% alfalfa, 2,
                       ifelse(x %in% non_ag, 3, 4)
                )
         )
  )
})

## ----fig.width=7, fig.height=5, fig.align="center"----------------------------
terra::plot(cdl_reclass, axes = FALSE, main = "Reclassified Landscape")

## -----------------------------------------------------------------------------
# Get the USDA NASS confusion matrix for Utah in 2008
trans_mat_08 <- readRDS(system.file("extdata/trans_mat_08.rds", 
                                    package = "cdlsim"))

## -----------------------------------------------------------------------------
print(trans_mat_08)

## -----------------------------------------------------------------------------
# Simulate the Bear Lake watershed 5 times
set.seed(1891)

bl_sim <- simulate_raster_patch(original_raster = cdl_reclass,
                      transition_matrix = trans_mat_08,
                      non_trans = c(0), iterations = 5)


## ----fig.width=7, fig.height=5, fig.align="center"----------------------------
# look at the simulations
terra::plot(bl_sim)

## -----------------------------------------------------------------------------
# Calculate Simpson's Diversity on 100 simulations
bl_sidi <- landscapemetrics::lsm_l_sidi(bl_sim) |>
  dplyr::rename(sim_value = value)


# Calculate Shannon's and Simpson on the 1000 iter sims and og models
og_sidi <- landscapemetrics::lsm_l_sidi(cdl_reclass) |>
  dplyr::rename(og_value = value)


# combine the og and sim results into one dataframe
div_combo_bl <- dplyr::full_join(bl_sidi, og_sidi, dplyr::join_by(metric)) |>
  dplyr::select(metric, sim_value, og_value)


## -----------------------------------------------------------------------------
# Print the tibble of the simulated versus original landscape values
print(div_combo_bl)

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

transition_function_single <- function(cell_values, transition_matrix, row_names, col_names) {
    .Call(`_cdlsim_transition_function_single`, cell_values, transition_matrix, row_names, col_names)
}

transition_function <- function(cell_values, transition_matrix, row_names, col_names, iterations) {
    .Call(`_cdlsim_transition_function`, cell_values, transition_matrix, row_names, col_names, iterations)
}

transition_patches <- function(cell_values, neg_values, trans_vecs, iterations) {
    .Call(`_cdlsim_transition_patches`, cell_values, neg_values, trans_vecs, iterations)
}

transition_patches_random <- function(cell_values, trans_vecs, iterations) {
    .Call(`_cdlsim_transition_patches_random`, cell_values, trans_vecs, iterations)
}

utils::globalVariables("total")

#' Function to Read in and upzip confusion matrix files from the web
#'
#' @param years The years of data that you want to be downloaded.
#' @param temp_dir The file name you want the extracted files to be stored in.
#' @returns The raw excel books containing the confusion matrix data for
#'    every US state's USDA Crop Land Data Layer.
#' @importFrom utils head tail download.file unzip
#' @export
download_cdl_mat_files <- function(years, temp_dir = "extracted_files") {
  base_url <- "https://www.nass.usda.gov/Research_and_Science/Cropland/docs/"
  file_pattern <- "CDL_%d_accuracy_assessments.zip"

  # Function to download and extract zip files
  download_and_extract <- function(url, temp_dir) {
    temp <- tempfile()
    download.file(url, temp, mode = "wb")
    unzip(temp, exdir = temp_dir)
    unlink(temp)
  }

  # Get the current working directory and combine it with the destination directory
  temp_dir_full <- file.path(getwd(), temp_dir)

  # Create directory to store the extracted files
  dir.create(temp_dir_full, showWarnings = FALSE)

  # Download and extract files for each year
  lapply(years, function(year) {
    file_url <- sprintf(paste0(base_url, file_pattern), year)
    message("Downloading and extracting: ", file_url)
    download_and_extract(file_url, temp_dir_full)
  })
}


#' Function to retrieve confusion matrix data for multiple states of interest
#' @param state_abbreviation A vector of two-letter abbreviations for US states.
#' @param file_path The path to the directory where files are stored
#'    (default is "inst/extdata/extracted_files").
#' @param verbose The stops the messages from printing to the console.
#' @return A named list where each element is a list of data frames representing
#'    confusion matrices for each state.
#' @importFrom dplyr intersect union
#' @importFrom stats na.omit
#' @import readxl
get_mat_data_dep <- function(state_abbreviation, file_path = "inst/extdata/extracted_files", verbose = FALSE) {
  # Ensure all state abbreviations are uppercase
  state_abbreviation <- toupper(state_abbreviation)

  # Use system.file() to correctly locate the installed files
  extracted_dir <- system.file("extdata/extracted_files", package = "cdlsim")

  # List of files to process (assuming files are already extracted)
  file_list <- list.files(path = extracted_dir, recursive = TRUE, full.names = TRUE, pattern = "\\.xlsx$")

  # Function to subset a data frame
  subset_data_frame <- function(df) {
    df[1:256, 3:258]
  }

  # Internal function to process files for a single state abbreviation
  process_state_files <- function(state_abbrev) {
    # Initialize a list to store the data for this state
    state_data <- list()

    # Create the regex patterns dynamically using the state abbreviation
    patterns <- list(
      sprintf("CDL_200[8-9]_accuracy_assessments/NASS_CDL_%s[0-9]{2}_accuracy_30m\\.xlsx$", state_abbrev),
      sprintf("CDL_201[0-5]_accuracy_assessments/NASS_CDL_%s[0-9]{2}_accuracy\\.xlsx$", state_abbrev),
      sprintf("CDL_2016_accuracy_assessments/unbuffered_validation/NASS_CDL_%s16_accuracy_unbuf\\.xlsx$", state_abbrev),
      sprintf("CDL_2017_accuracy_assessments/NASS_CDL_%s17_accuracy\\.xlsx$", state_abbrev),
      sprintf("NASS_CDL_%s(1[8-9]|2[0-3])_accuracy\\.xlsx$", state_abbrev) # 2018-2023
    )
    # Loop through each file and process it
    for (file in file_list) {
      # Initialize data_file as NULL at the beginning of each iteration
      data_file <- NULL

      # Check if the file matches any of the patterns
      for (pattern in patterns) {
        if (grepl(pattern, file)) {
          data_file <- file
          break
        }
      }

      # Read the third sheet from the Excel file if a match is found
      if (!is.null(data_file) && file.exists(data_file)) {
        if (verbose) message("Processing: ", data_file)
        data <- readxl::read_excel(data_file, sheet = 3, .name_repair = "minimal")

        # Subset the data to the required range
        data <- data[1:256, 3:258]

        # Extract the year from the file name
        year_match <- sub(".*([0-9]{2})_accuracy.*\\.xlsx$", "\\1", data_file)
        year <- ifelse(length(year_match) > 0, year_match, "Unknown Year")

        # Store the data in the state_data list
        state_data[[year]] <- data
      } else if (verbose) {
        message("File does not match pattern or does not exist: ", file)
      }
    }

    return(state_data)
  }

  # Use Map to apply process_state_files to each state abbreviation
  all_states_data <- Map(process_state_files, state_abbreviation)

  # Set names of the list as the state abbreviations for easy reference
  names(all_states_data) <- state_abbreviation

  # Return the combined list of all states' data
  return(all_states_data)
}


#' Function to retrieve confusion matrix data for multiple states of interest
#'
#' @param state_abbreviation A vector of two-letter abbreviations for US states.
#' @param file_path The path to the directory where files are stored
#'    (default is "inst/extdata/extracted_files").
#' @param verbose The stops the messages from printing to the console.
#' @return A named list where each element is a list of data frames representing
#'    confusion matrices for each state.
#' @importFrom dplyr intersect union
#' @importFrom stats na.omit
#' @import readxl
#' @export
#' @examples
#' # example code
#' # Get data from UT in 2008
#' ut_mat_data <- get_mat_data(c("UT"))
get_mat_data <- function(state_abbreviation, file_path = "inst/extdata/extracted_files", verbose = FALSE) {
  state_abbreviation <- toupper(state_abbreviation)

  # Track whether this is a direct file input
  single_file_input <- FALSE

  # Resolve system file path if default
  if (file_path == "inst/extdata/extracted_files") {
    file_path <- system.file("extdata/extracted_files", package = "cdlsim")
  }

  # Detect single file input
  if (file.exists(file_path) && grepl("\\.xlsx$", file_path)) {
    file_list <- file_path
    single_file_input <- TRUE
  } else {
    file_list <- list.files(path = file_path, recursive = TRUE, full.names = TRUE, pattern = "\\.xlsx$")
  }

  subset_data_frame <- function(df) {
    df[1:256, 3:258]
  }

  process_state_files <- function(state_abbrev) {
    state_data <- list()


    if (single_file_input && file.exists(file_list)) {
      if (verbose) message("Using provided file directly: ", file_list)
      data <- readxl::read_excel(file_list, sheet = 3, .name_repair = "minimal")
      data <- subset_data_frame(data)

      return(data)
    }

    # Otherwise use pattern matching
    patterns <- list(
      sprintf("CDL_200[8-9]_accuracy_assessments/NASS_CDL_%s[0-9]{2}_accuracy_30m\\.xlsx$", state_abbrev),
      sprintf("CDL_201[0-5]_accuracy_assessments/NASS_CDL_%s[0-9]{2}_accuracy\\.xlsx$", state_abbrev),
      sprintf("CDL_2016_accuracy_assessments/unbuffered_validation/NASS_CDL_%s16_accuracy_unbuf\\.xlsx$", state_abbrev),
      sprintf("CDL_2017_accuracy_assessments/NASS_CDL_%s17_accuracy\\.xlsx$", state_abbrev),
      sprintf("NASS_CDL_%s(1[8-9]|2[0-3])_accuracy\\.xlsx$", state_abbrev),
      sprintf("NASS_CDL_%s[0-9]{2}_accuracy_30m\\.xlsx$", state_abbrev)  # catch-all
    )

    for (file in file_list) {
      data_file <- NULL
      file_basename <- basename(file)

      for (pattern in patterns) {
        if (grepl(pattern, file) || grepl(pattern, file_basename)) {
          data_file <- file
          break
        }
      }

      if (!is.null(data_file) && file.exists(data_file)) {
        if (verbose) message("Processing: ", data_file)
        data <- readxl::read_excel(data_file, sheet = 3, .name_repair = "minimal")
        data <- subset_data_frame(data)

        year_match <- sub(".*([0-9]{2})_accuracy.*\\.xlsx$", "\\1", basename(data_file))
        year <- ifelse(length(year_match) > 0, year_match, "Unknown Year")

        state_data[[year]] <- data
      } else if (verbose) {
        message("File does not match pattern or does not exist: ", file)
      }
    }

    return(state_data)
  }


  if (single_file_input) {
    return(process_state_files(state_abbreviation[1]))
  }


  all_states_data <- Map(process_state_files, state_abbreviation)
  names(all_states_data) <- state_abbreviation

  return(all_states_data)
}


## update for the case when the list of categories is the same as the length list of
# original classes
#' Function to format the confusion matrices as transition matrices
#'
#' @param df_list A list of lists of data frames extracted using get_mat_data(). Each sublist should contain two or more data frames to be summed.
#' @param categories A list of categories defining the numbers between 1 and 256.
#' @returns A list of data frames where row names represent pixels that will
#'    transition and column names represent the class they will transition to.
#' @import dplyr
#' @importFrom stats na.omit
get_trans_mat_dep <- function(df_list, categories) {
  # Check if the input is a list of lists of data frames and if it contains data frames
  if (length(df_list) == 0 || !all(sapply(df_list, function(x) all(sapply(x, is.data.frame))))) {
    stop("Input must be a list of lists, each containing data frames")
  }

  # Flatten the categories and check for duplicates and completeness
  all_category_numbers <- unlist(categories)

  # Check for duplicates
  if (any(duplicated(all_category_numbers))) {
    stop("There are duplicate numbers in the list of vectors describing the categories")
  }

  # Check for completeness (1 to 256)
  if (!all(sort(all_category_numbers) == 1:256)) {
    stop("Not all numbers between 1 and 256 are included in the list of vectors describing the categories")
  }

  # Initialize a list to store the results by year
  result_list <- vector("list", length(df_list[[1]]))
  names(result_list) <- names(df_list[[1]]) # Assign names for each year

  # Iterate over each year (position in the sublists)
  for (k in seq_along(result_list)) {
    # Sum the matrices from all data frames for the same year across states
    sum_mat <- Reduce(`+`, lapply(df_list, function(x) as.matrix(x[[k]])))

    # Convert the summed matrix back to a data frame
    df <- as.data.frame(sum_mat)

    # Initialize an empty matrix to store the category sums
    category_matrix <- matrix(0, nrow = length(categories), ncol = length(categories))

    # Sum rows and columns for each category based on the provided vectors
    for (i in seq_along(categories)) {
      for (j in seq_along(categories)) {
        category_rows <- df %>%
          slice(categories[[i]]) %>%
          summarize(across(everything(), sum))

        category_sum <- category_rows %>%
          select(all_of(categories[[j]])) %>%
          summarize(across(everything(), sum))

        category_matrix[i, j] <- sum(category_sum)
      }
    }

    # Convert the matrix to a data frame
    df_trans <- as.data.frame(category_matrix)

    # Calculate total and proportions
    df_trans <- df_trans |>
      mutate(total = rowSums(across(everything()))) |>
      mutate(across(-total, ~ . / total))

    # Update row names to be the negative value of the row number
    rownames(df_trans) <- as.character((1:nrow(df_trans)))

    # Update column names to be the column numbers
    colnames(df_trans) <- as.character(1:ncol(df_trans))

    # Remove the last column
    df_trans <- df_trans[, -ncol(df_trans)]


    # Insert a row and column of zeros at the first positions
    n <- nrow(df_trans) + 1  # Updated number of rows and columns
    updated_matrix <- matrix(0, nrow = n, ncol = n)
    updated_matrix[-1, -1] <- as.matrix(df_trans)  # Fill with existing values

    # Convert the updated matrix back to a data frame
    updated_matrix[1, 1] <- 1
    df_trans_updated <- as.data.frame(updated_matrix)

    # Update row and column names for the new matrix
    rownames(df_trans_updated) <- as.character(0:(n - 1))
    colnames(df_trans_updated) <- as.character(0:(n - 1))

    # Store the resulting data frame for this year
    result_list[[k]] <- df_trans_updated
  }

  # Return the list of results
  return(result_list)
}


#' Function to format the confusion matrices as transition matrices
#'
#' @param df_list A single data frame or a list of lists of data frames extracted using get_mat_data().
#'                If a list, each sublist should contain two or more data frames to be summed.
#' @param categories A list of categories defining the numbers between 1 and 256.
#' @returns A list of data frames where row names represent pixels that will
#'    transition and column names represent the class they will transition to.
#' @import dplyr
#' @export
#' @examples
#' # make the data frame
#' bl_mat_data <- get_mat_data(c("UT"))
#'
#' # List of categories with their corresponding vectors
#' # Define the values that represent our classes of interest
#' non_ag <- c(61:65, 81:83, 87:88, 92, 111:112, 121:124, 131, 141:143, 152, 176, 181, 190, 195)
#' alfalfa <- c(36:37)
#' major_ag <- c(1, 2, 5, 12, 13, 22:24, 26, 225:226, 228, 234, 236, 238:241, 254)
#' all_numbers <- 1:256
#' ag <- setdiff(all_numbers, c(non_ag, alfalfa, major_ag))
#'
#' cat_5 <- list(non_ag = non_ag, ag = ag, alfalfa = alfalfa, major_ag = major_ag)
#'
#' # get the confusion matrix for just 2008
#' trans_mat_5 <- get_trans_mat(bl_mat_data, cat_5)
get_trans_mat <- function(df_list, categories) {
  # If input is a single data frame, convert it to a list containing one list
  if (is.data.frame(df_list)) {
    df_list <- list(list(df_list))
  }

  # If input is a list of data frames (not nested), wrap it in an additional list
  if (is.list(df_list) && all(sapply(df_list, is.data.frame))) {
    df_list <- list(df_list)
  }

  # Check if the input is a properly formatted list of lists of data frames
  if (length(df_list) == 0 || !all(sapply(df_list, function(x) all(sapply(x, is.data.frame))))) {
    stop("Input must be a single data frame, a list of data frames, or a list of lists containing data frames")
  }

  # Flatten the categories and check for duplicates and completeness
  all_category_numbers <- unlist(categories)

  # Check for duplicates
  if (any(duplicated(all_category_numbers))) {
    stop("There are duplicate numbers in the list of vectors describing the categories")
  }

  # Check for completeness (1 to 256)
  if (!all(sort(all_category_numbers) == 1:256)) {
    stop("Not all numbers between 1 and 256 are included in the list of vectors describing the categories")
  }

  # Initialize a list to store the results by year
  result_list <- vector("list", length(df_list[[1]]))
  names(result_list) <- names(df_list[[1]]) # Assign names for each year if applicable

  # Iterate over each year (position in the sublists)
  for (k in seq_along(result_list)) {
    # Sum the matrices from all data frames for the same year across states
    sum_mat <- Reduce(`+`, lapply(df_list, function(x) as.matrix(x[[k]])))

    # Convert the summed matrix back to a data frame
    df <- as.data.frame(sum_mat)

    # Initialize an empty matrix to store the category sums
    category_matrix <- matrix(0, nrow = length(categories), ncol = length(categories))

    # Sum rows and columns for each category based on the provided vectors
    for (i in seq_along(categories)) {
      for (j in seq_along(categories)) {
        category_rows <- df %>%
          slice(categories[[i]]) %>%
          summarize(across(everything(), sum))

        category_sum <- category_rows %>%
          select(all_of(categories[[j]])) %>%
          summarize(across(everything(), sum))

        category_matrix[i, j] <- sum(category_sum)
      }
    }

    # Convert the matrix to a data frame
    df_trans <- as.data.frame(category_matrix)

    # Calculate total and proportions
    df_trans <- df_trans |>
      mutate(total = rowSums(across(everything()))) |>
      mutate(across(-total, ~ . / total))

    # Update row names to be the negative value of the row number
    rownames(df_trans) <- as.character((1:nrow(df_trans)))

    # Update column names to be the column numbers
    colnames(df_trans) <- as.character(1:ncol(df_trans))

    # Remove the last column
    df_trans <- df_trans[, -ncol(df_trans)]

    # Insert a row and column of zeros at the first positions
    n <- nrow(df_trans) + 1  # Updated number of rows and columns
    updated_matrix <- matrix(0, nrow = n, ncol = n)
    updated_matrix[-1, -1] <- as.matrix(df_trans)  # Fill with existing values

    # Convert the updated matrix back to a data frame
    updated_matrix[1, 1] <- 1
    df_trans_updated <- as.data.frame(updated_matrix)

    # Update row and column names for the new matrix
    rownames(df_trans_updated) <- as.character(0:(n - 1))
    colnames(df_trans_updated) <- as.character(0:(n - 1))

    # Store the resulting data frame for this year
    result_list[[k]] <- df_trans_updated
  }

  # Return the list of results
  return(result_list)
}



get_trans_mat_simple <- function(df_list, categories) {
  # Check if the input is a list of lists of data frames and if it contains data frames
  if (length(df_list) == 0 || !all(sapply(df_list, function(x) all(sapply(x, is.data.frame))))) {
    stop("Input must be a list of lists, each containing data frames")
  }

  # Flatten the categories and check for duplicates and completeness
  all_category_numbers <- unlist(categories)

  # Check for duplicates
  if (any(duplicated(all_category_numbers))) {
    stop("There are duplicate numbers in the list of vectors describing the categories")
  }

  # Check for completeness (1 to 256)
  if (!all(sort(all_category_numbers) == 1:256)) {
    stop("Not all numbers between 1 and 256 are included in the list of vectors describing the categories")
  }

  # Initialize a list to store the results by year
  result_list <- vector("list", length(df_list[[1]]))
  names(result_list) <- names(df_list[[1]]) # Assign names for each year

  # Iterate over each year (position in the sublists)
  for (k in seq_along(result_list)) {
    # Sum the matrices from all data frames for the same year across states
    sum_mat <- Reduce(`+`, lapply(df_list, function(x) as.matrix(x[[k]])))

    # Initialize an empty matrix to store the category sums
    category_matrix <- matrix(0, nrow = length(categories), ncol = length(categories))

    # Sum rows and columns for each category based on the provided vectors
    for (i in seq_along(categories)) {
      for (j in seq_along(categories)) {
        category_rows <- sum_mat[categories[[i]], , drop = FALSE]
        category_matrix[i, j] <- sum(category_rows[, categories[[j]], drop = FALSE])
      }
    }

    # Add totals row and column
    total_row <- rowSums(category_matrix)
    total_col <- colSums(category_matrix)
    total_sum <- sum(total_row) # Grand total

    # Expand the matrix to include totals
    category_matrix <- rbind(category_matrix, total_col)
    category_matrix <- cbind(category_matrix, c(total_row, total_sum))

    # Convert to a data frame
    df_trans <- as.data.frame(category_matrix)

    # Set row and column names
    rownames(df_trans) <- c(paste0("Cat_", seq_along(categories)), "Total")
    colnames(df_trans) <- c(paste0("Cat_", seq_along(categories)), "Total")

    # Store the resulting data frame for this year
    result_list[[k]] <- df_trans
  }

  # Return the list of results
  return(result_list)
}

#' Function to resize patches in a SpatRaster using the terra and landscapemetrics packages
#'
#' @param raster The SpatRaster object representing the categorical data.
#' @returns A SpatRaster where each patch is defined as a unique value and each
#'    class has its own layer in the spatraster.
#' @import landscapemetrics
find_patches <- function(raster) {

  # Get patches using landscapemetrics::get_patches
  patches_list <- landscapemetrics::get_patches(raster)
  result_rast <- terra::rast(patches_list[[1]])

  # name the layers based on their associated class
  # Extract numeric value
  og_names <- names(result_rast)
  number <- as.numeric(gsub("class_", "", og_names))
  class_names <- c(number)
  names(result_rast) <- class_names

  return(result_rast)

}

# update this function to allow transition matrices that don't have col names
# i.e. the class or col name would then be = (column index - 1)

#' Function to generate transition vectors for negative cell values
#'
#' @param r The SpatRaster object from get_patches with one layer for each
#'    class.
#' @param transition_matrix The transition matrix you want to use.
#' @param iterations The number of iterations desired.
#' @returns A matrix of vector for each patch in each layer.
generate_transition_vectors <- function(r, transition_matrix, iterations = 10) {
  # Initialize a list to store the transition vectors for each unique patch ID
  simulated_values <- list()

  # Extract unique non-NA values for each layer and store in a list
  unique_values_list <- lapply(1:terra::nlyr(r), function(i) {
    as.vector(na.omit(unique(terra::values(r[[i]]))))
  })

  # Set the names of the list elements to match the layer names
  names(unique_values_list) <- names(r)

  # Loop through each list element in unique_values_list
  for (layer_name in names(unique_values_list)) {
    neg_values <- unique_values_list[[layer_name]]

    # Initialize an empty matrix to store the transition vectors for this layer
    layer_matrix <- matrix(nrow = length(neg_values), ncol = iterations)
    rownames(layer_matrix) <- neg_values

    # Loop through each unique value in the current list element
    for (i in seq_along(neg_values)) {
      neg_val <- neg_values[i]

      # Get the transition probabilities for the current layer name from confusion_matrix
      probabilities <- transition_matrix[which(rownames(transition_matrix) == layer_name), ]

      #Get col names to sample from transition matrix
      col_names <- as.numeric(colnames(transition_matrix))

      # Generate the transition vector by sampling with replacement
      new_values <- sample(col_names, size = iterations, replace = TRUE, prob = probabilities)

      # Store the transition vector in the corresponding row of the matrix
      layer_matrix[i, ] <- new_values
    }

    # Store the matrix in the list, with the layer name as the key
    simulated_values[[layer_name]] <- layer_matrix
  }

  return(simulated_values)
}

# might need to change this function since we will alwyays have col/row names

#' Function to modify transition matrix based on user input.
#'
#' @param mat The transition matrix the you want to update.
#' @param indices The vector of classes you want to remain unsimulated.
#' @returns A transition matrix modified to not transition some classes.
modify_matrix <- function(mat, indices = 0) {
  # Check if row and column names exist
  if (!is.null(rownames(mat)) && !is.null(colnames(mat)))  {

    # makes sure the input classes are characters
    indices = as.character(indices)

    # Create a copy of the matrix
    modified_mat <- mat

    # Loop through each name
    for (index in indices) {
      # Find the corresponding row and column
      row_idx <- which(rownames(mat) == index)
      col_idx <- which(colnames(mat) == index)

      # Set the diagonal element to 1
      modified_mat[row_idx, col_idx] <- 1

      # Set other elements in the row and column to 0
      modified_mat[row_idx, -col_idx] <- 0
      modified_mat[-row_idx, col_idx] <- 0
    }


  }else{
    # Create a copy of the input matrix
    modified_mat <- mat

    # Loop through each index in the vector
    for (index in indices) {
      # Set the main diagonal value to 1 for the specified index
      modified_mat[(index+1), (index+1)] <- 1

      # Set the corresponding row and column values to 0, except the diagonal
      modified_mat[(index+1), -(index+1)] <- 0
      modified_mat[-(index+1), (index+1)] <- 0
    }


  }

  return(modified_mat)
}

#' Function to resize patches in a SpatRaster using the terra and landscapemetrics packages
#'
#' @param patched_raster The SpatRaster object representing the categorical data.
#' @param og_raster The values of the classes of interest, default is "all".
#' @returns A SpatRaster where each patch is defined as a unique value and each
#'    class has its own layer in the spatraster.
collapse_and_combined <- function(patched_raster, og_raster) {

  # collapse the patched raster to the first cell value that is not NA
  collapsed_layer <- terra::app(patched_raster, fun = function(x) {
    return(x[which(!is.na(x))[1]])
  })

  # make two layer raster with og and collapsed layers
  combo <- c(og_raster, collapsed_layer)

  return(combo)
}

#' Function to simulate tagged patch values in a single layer spatraster.
#'
#' @param original_raster The SpatRaster object representing the categorical data.
#' @param transition_matrix Transition matrix values that define transitions
#'    based on class values.
#' @param iterations The number of simulations to be performed.
#' @param non_trans Class(es) that the user would like to remain unsimulated.
#'    Defualt is class 0 since this is the background class.
#' @returns A SpatRaster where the patches values have been simulated.
#' @import landscapemetrics
#' @export
#' @examples
#' library(terra)
#'
#' r <- rast(nrows = 20, ncols = 20)
#' values(r) <- rep(1:3, length.out = ncell(r))
#'
#' trans_mat <- matrix(1/3, 3, 3)
#' rownames(trans_mat) <- colnames(trans_mat) <- c("1", "2", "3")
#'
#' simulate_raster_patch(r, trans_mat, iterations = 1)
simulate_raster_patch <- function(original_raster,
                                  transition_matrix,
                                  non_trans = 0,
                                  iterations = 10) {

  # Ensure objects are the correct class
  transition_matrix <- as.matrix(transition_matrix)
  iters <- as.integer(iterations)

  # Update any classes that are set to not transition (still need to check with simulation)
  transition_matrix <- modify_matrix(mat = transition_matrix,
                                     indices = non_trans)

  # Find the patches in the og layer
  patched_raster <- find_patches(original_raster)

  # generate transition vectors
  vecs <- generate_transition_vectors(r = patched_raster, transition_matrix = transition_matrix, iterations = iters)

  # Collapse the patched raster to a single layer and combined with input
  tagged_raster <- collapse_and_combined(patched_raster, og_raster = original_raster)

  # Initialize the simulation with the original raster
  current_raster <- original_raster

  # create the new simulated raster
  current_raster <- terra::app(tagged_raster, fun = transition_patches_random,
                               trans_vecs = vecs,
                               iterations = iters)

  return(current_raster)

}

test_that("sim diversity does not exceed max of input raster", {
  # test that in the case where the input raster is at max diversity that the
  # the simulation's max diversity doesn't exceed the input.

  #### Make the large max diversity raster
  # Set parameters
  n_classes <- 3        # Number of classes
  square_size <- 30     # Each square is 30x30 pixels
  n_squares <- 9        # 9x9 grid of squares

  # Calculate raster dimensions
  ncol <- square_size * n_squares   # Width to fit 9 squares across
  nrow <- square_size * n_squares   # Height to fit 9 squares down

  # Create an empty raster
  rbig <- terra::rast(ncol = ncol, nrow = nrow)
  terra::values(rbig) <- NA  # Initialize with NA values

  # Assign alternating class values to each square in a checkerboard pattern
  for (i in 1:n_squares) {
    for (j in 1:n_squares) {
      # Define square boundaries
      row_start <- (i - 1) * square_size + 1
      row_end <- i * square_size
      col_start <- (j - 1) * square_size + 1
      col_end <- j * square_size

      # Calculate class value based on row and column position, alternating classes
      class_value <- ((i + j - 2) %% n_classes) + 1  # Alternates classes 1, 2, and 3 in a checkerboard pattern

      # Assign values to the defined square region
      rbig[row_start:row_end, col_start:col_end] <- class_value
    }
  }

  # Define confusion matrix, background not transitioning
  conf_mat <- matrix(1/3, nrow = 3, ncol = 3)

  # Assign row and column names
  rownames(conf_mat) <- c( "1", "2", "3")
  colnames(conf_mat) <- c( "1", "2", "3")

  # create 5 simulations with class not transitioning
  div_sim <- simulate_raster_patch(original_raster = rbig,
                                     transition_matrix = conf_mat,
                                     iterations = 10)

  # calculate Shannon's diversity using the landscapemetrics package
  shdi_sim <- landscapemetrics::lsm_l_sidi(div_sim)
  interval <- quantile(shdi_sim$value, c(0.025, 0.5, 0.975))
  shdi_sim_int <- dplyr::as_tibble(interval) |>
    dplyr::mutate(quantile = c(2.5, 50, 97.5)) |>
    dplyr::filter(quantile == 97.5) |>
    dplyr::pull(value)

  shdi_og <- landscapemetrics::lsm_l_shdi(rbig) |>
    dplyr::pull(value)

  # expect that the og div value is always bigger than the max sim value
  expect_true(shdi_og > shdi_sim_int)
})

test_that("Non-transitions maintain pixel count", {
  # test that the simulated rasters have the correct number cells in a class.

  #### Make the smaller max diversity raster
  # Define raster dimensions and classes
  ncol <- 90
  nrow <- 90
  classes <- 3

  # Initialize a SpatRaster with the desired dimensions
  r <- terra::rast(ncol = ncol, nrow = nrow, vals = NA)

  # Assign values in 30x30 blocks for each class
  r[1:30, 1:30] <- 1  # Top-left 30x30 block
  r[1:30, 31:60] <- 2  # Top-center 30x30 block
  r[1:30, 61:90] <- 3  # Top-right 30x30 block

  r[31:60, 1:30] <- 2  # Middle-left 30x30 block
  r[31:60, 31:60] <- 3  # Middle-center 30x30 block
  r[31:60, 61:90] <- 1  # Middle-right 30x30 block

  r[61:90, 1:30] <- 3  # Bottom-left 30x30 block
  r[61:90, 31:60] <- 1  # Bottom-center 30x30 block
  r[61:90, 61:90] <- 2  # Bottom-right 30x30 block

  # Define confusion matrix, background not transitioning
  conf_mat <- matrix(1/3, nrow = 3, ncol = 3)

  # Assign row and column names
  rownames(conf_mat) <- c( "1", "2", "3")
  colnames(conf_mat) <- c( "1", "2", "3")
  conf_mat[1, ] <- 0
  conf_mat[ , 1] <- 0
  conf_mat[1, 1] <- 1


  # create 5 simulations with class not transitioning
  class_sim <- simulate_raster_patch(original_raster = r,
                                     transition_matrix = conf_mat,
                                     iterations = 1)

  # check that freq of input out sims are the same for class 1
  og_counts <- terra::freq(r)
  class_1_og <- og_counts[og_counts$value == 1, 3]

  sim_counts <- terra::freq(class_sim)
  class_1_sim <- sim_counts[sim_counts$value == 1, 3]

  # expect that no values in the tagged raster are negative
  expect_equal(class_1_og, class_1_sim)
})

test_that("no pixels are dropped", {
  # Still need to check this test
  # Create an empty raster with dimensions 100 by 100
  r <- terra::rast(nrows=100, ncols=100, xmin=0, xmax=100, ymin=0, ymax=100)

  # Initialize all cells with class 1
  terra::values(r) <- 1

  # Define coordinates for the two 30x30 squares of class 2
  # First square (top-left corner at (10, 10))
  for (i in 10:39) {  # 10 to 39 makes a 30x30 square
    for (j in 10:39) {
      r[i, j] <- 2
    }
  }

  # Second square (top-left corner at (50, 50))
  for (i in 50:79) {  # 50 to 79 makes a 30x30 square
    for (j in 50:79) {
      r[i, j] <- 2
    }
  }

  # Make conf mat with 1s on diagonal
  n <- 3
  conf_mat <- matrix(0, nrow = n, ncol = n)
  diag(conf_mat) <- 1

  # Assign row and column names
  rownames(conf_mat) <- c("1", "2", "3")
  colnames(conf_mat) <- c("1", "2", "3")
  conf_mat

  patch_sim <- simulate_raster_patch(original_raster = r,
                                       transition_matrix = conf_mat,
                                       iterations = 1)


  mismatch_mask <- r != patch_sim


  # expect that no values in the tagged raster are negative
  expect_equal(sum(terra::values(mismatch_mask)), 0)
})

---
title: "cdlsim-simulations"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{cdlsim-simulations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cdlsim)
```

## The USDA Cropland Data Layer 

One of the primary tools used in the United States to monitor agricultural land is the National Agricultural Statistics Service (NASS) Cropland Data Layer (CDL), an annual map created by the United States Department of Agriculture (USDA) from satellite images. While the CDL is highly accurate, the data product is still prone to misclassify certain land use types. These challenges can affect the reliability of any metrics calculated using the data. To address this, we have developed `cdlsim`, which simulates land use changes to quantify the sensitivity of land use metrics to variability in the input data. We demonstrate the general uses of our package through two examples: a simple example using a mock landscape and a real world example in the Bear Lake watershed in Utah.


## Package Functions

The two major purposes of our package are to download and format the transition matrices and to simulate the CDL data provided as the input. To accomplish the former, there are three main functions: `download_cdl_mat_files`, `get_mat_data`, and `get_trans_mat`. The first function downloads and unzips folders containing the confusion matrices for all the US states using their zip file's URL. The range of years of data to download and the file path to save the corresponding Excel files can be specified in the arguments. The `get_mat_data` and `get_trans_mat` functions then take the previously downloaded data and extract the files for the states supplied. These functions also read in the tables (formatted as Excel spreadsheets) as matrices, allowing for easier manipulation, formatting, and improving compatibility with the simulation functions.

The second major purpose of `cdlsim` is the simulation function `simulate_raster_rcpp`. Here we simulate class values at the patch level maintaining the spatial integrity and and structure of the landscape. This function takes as input a `SpatRaster`, a transition matrix, and the desired number of iterations. Several additional arguments can be optionally set. For example, in many cases, it is reasonable to prevent specific pixel values, such as those representing lakes or streams, from transitioning. These values can be included in the `non_trans` argument, ensuring that the corresponding pixels retain their original class. By default, this argument prevents the background class (0) from transitioning, as this class represents areas with no data. Additionally to accommodate larger study areas the simulation function includes a memory-efficient option that is able to handle larger raster files. The user can activate the memory efficient simulation version using the `Fast` argument. By default, `Fast = TRUE`, which is recommended for smaller study areas, such as counties size regions.


## What is a Patch?

A patch is defined as a group of connected neighboring cells from the same class. For simulation purposes, adjacent or diagonally connected pixels of the same class are treated as a single patch (i.e., queen adjacency). This definition follows similarly to that of patches in the `landscapemetrics` ensuring compatibility. 

## Installation

You can install the development version of `cdlsim` from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("burgerhaley97/cdlsim")
```

## The Simple Example

This is a basic example which shows how a simple landscape would behave in our 
simulation. We start by building a SpatRaster with three different classes each 
with three 90 by 90 pixel squares.

```{r}
# Define raster dimensions and classes
ncol <- 90
nrow <- 90
classes <- 3

# Initialize a SpatRaster with the desired dimensions
r_small <- terra::rast(ncol = ncol, nrow = nrow, vals = NA)

# Assign values in 30x30 blocks for each class
r_small[1:30, 1:30] <- 1  
r_small[1:30, 31:60] <- 2  
r_small[1:30, 61:90] <- 3 

r_small[31:60, 1:30] <- 2  
r_small[31:60, 31:60] <- 3  
r_small[31:60, 61:90] <- 1 

r_small[61:90, 1:30] <- 3  
r_small[61:90, 31:60] <- 1  
r_small[61:90, 61:90] <- 2  

```

```{r fig.width=7, fig.height=5, fig.align="center"}
# Show original landscape before simulation
terra::plot(r_small, axes = FALSE, main = "Simple Landscape")

```



Next we define the transition matrix which describes the probability that each 
class has of transitioning to the other classes during simulation. Note that the
row names represent the original landscape's classes and the column 
names represent the simulated class values. For example, the first entry below 
describes the chance that class one patches will transition to class one during simulation.


```{r}
# Define confusion matrix, background not transitioning
conf_mat <- matrix(1/3, nrow = 3, ncol = 3)

# Assign row and column names
rownames(conf_mat) <- c("1", "2", "3")
colnames(conf_mat) <- c("1", "2", "3")

# Print the transition matrix 
print(conf_mat)

```

Finally we simulate the original landscape five times and plot the results. 

```{r}
# Set a random seed for reproducibility 
set.seed(1891)

# Simulate the example landscape 5 times
simple_sim <- simulate_raster_patch(original_raster = r_small,
                                       transition_matrix = conf_mat,
                                       iterations = 5)
```


```{r fig.width=7, fig.height=5, fig.align="center"}
# Plot the results
terra::plot(simple_sim, axes = FALSE)
```


## The Bear Lake Watershed

Here we use the R package `landscapemetrics` to implement popular FRAGSTATS style metrics to demonstrate how `cdlsim` can be used to quantify metric sensitivity to random perturbations introduced through simulation. To illustrate, we use the Cropland Data Layer (CDL) for the Bear Lake watershed in 2008, along with the associated confusion matrix for that year.


```{r}
# Load the raster
bl_raster <- terra::rast(system.file("extdata/cropped_raster.tif", package="cdlsim"))
```


```{r fig.width=7, fig.height=5, fig.align="center"}
terra::plot(bl_raster, axes = FALSE, main = "Bear Lake Watershed")
```

The CDL data for the Bear Lake watershed is reclassified before simulation. 

```{r}
# Define the values that represent our classes of interest
non_ag <- c(61:65, 81:83, 87:88, 92, 111:112, 121:124, 131, 141:143, 152, 176, 181, 190, 195)
alfalfa <- c(36:37)
major_ag <- c(1, 2, 5, 12, 13, 22:24, 26, 225:226, 228, 234, 236, 238:241, 254)
all_numbers <- 1:256
ag <- setdiff(all_numbers, c(non_ag, alfalfa, major_ag))

# reclassify into background = 0, major-ag = 1, alfalfa = 2, non-ag = 3, ag = 4
cdl_reclass <- terra::app(bl_raster, fun = function(x) {
  ifelse(is.na(x), 0,
         ifelse(x %in% major_ag, 1,
                ifelse(x %in% alfalfa, 2,
                       ifelse(x %in% non_ag, 3, 4)
                )
         )
  )
})
```


```{r fig.width=7, fig.height=5, fig.align="center"}
terra::plot(cdl_reclass, axes = FALSE, main = "Reclassified Landscape")
```


```{r}
# Get the USDA NASS confusion matrix for Utah in 2008
trans_mat_08 <- readRDS(system.file("extdata/trans_mat_08.rds", 
                                    package = "cdlsim"))
```


```{r}
print(trans_mat_08)
```


```{r}
# Simulate the Bear Lake watershed 5 times
set.seed(1891)

bl_sim <- simulate_raster_patch(original_raster = cdl_reclass,
                      transition_matrix = trans_mat_08,
                      non_trans = c(0), iterations = 5)

```

```{r fig.width=7, fig.height=5, fig.align="center"}
# look at the simulations
terra::plot(bl_sim)
```


```{r}
# Calculate Simpson's Diversity on 100 simulations
bl_sidi <- landscapemetrics::lsm_l_sidi(bl_sim) |>
  dplyr::rename(sim_value = value)


# Calculate Shannon's and Simpson on the 1000 iter sims and og models
og_sidi <- landscapemetrics::lsm_l_sidi(cdl_reclass) |>
  dplyr::rename(og_value = value)


# combine the og and sim results into one dataframe
div_combo_bl <- dplyr::full_join(bl_sidi, og_sidi, dplyr::join_by(metric)) |>
  dplyr::select(metric, sim_value, og_value)

```

Compare the simulated Simpson's Diversity Index values to the original landscape's values.

```{r}
# Print the tibble of the simulated versus original landscape values
print(div_combo_bl)

```

The `cdlsim` package provides a flexible and intuitive framework for simulating classification uncertainty in USDA Cropland Data Layer products at the patch level. Through both a simple example and a real-world application in the Bear Lake watershed, we demonstrate how `cdlsim` can be used to evaluate the sensitivity of landscape metrics to random perturbations in classification. The package supports transition matrices that are either user-defined or derived from USDA NASS published confusion matrices, offering valuable tools for researchers aiming to assess the robustness of land use analyses under data uncertainty.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patch_sim.R
\name{find_patches}
\alias{find_patches}
\title{Function to resize patches in a SpatRaster using the terra and landscapemetrics packages}
\usage{
find_patches(raster)
}
\arguments{
\item{raster}{The SpatRaster object representing the categorical data.}
}
\value{
A SpatRaster where each patch is defined as a unique value and each
class has its own layer in the spatraster.
}
\description{
Function to resize patches in a SpatRaster using the terra and landscapemetrics packages
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patch_sim.R
\name{simulate_raster_patch}
\alias{simulate_raster_patch}
\title{Function to simulate tagged patch values in a single layer spatraster.}
\usage{
simulate_raster_patch(
  original_raster,
  transition_matrix,
  non_trans = 0,
  iterations = 10
)
}
\arguments{
\item{original_raster}{The SpatRaster object representing the categorical data.}

\item{transition_matrix}{Transition matrix values that define transitions
based on class values.}

\item{non_trans}{Class(es) that the user would like to remain unsimulated.
Defualt is class 0 since this is the background class.}

\item{iterations}{The number of simulations to be performed.}
}
\value{
A SpatRaster where the patches values have been simulated.
}
\description{
Function to simulate tagged patch values in a single layer spatraster.
}
\examples{
library(terra)

r <- rast(nrows = 20, ncols = 20)
values(r) <- rep(1:3, length.out = ncell(r))

trans_mat <- matrix(1/3, 3, 3)
rownames(trans_mat) <- colnames(trans_mat) <- c("1", "2", "3")

simulate_raster_patch(r, trans_mat, iterations = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patch_sim.R
\name{modify_matrix}
\alias{modify_matrix}
\title{Function to modify transition matrix based on user input.}
\usage{
modify_matrix(mat, indices = 0)
}
\arguments{
\item{mat}{The transition matrix the you want to update.}

\item{indices}{The vector of classes you want to remain unsimulated.}
}
\value{
A transition matrix modified to not transition some classes.
}
\description{
Function to modify transition matrix based on user input.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_conf_mat.R
\name{get_mat_data_dep}
\alias{get_mat_data_dep}
\title{Function to retrieve confusion matrix data for multiple states of interest}
\usage{
get_mat_data_dep(
  state_abbreviation,
  file_path = "inst/extdata/extracted_files",
  verbose = FALSE
)
}
\arguments{
\item{state_abbreviation}{A vector of two-letter abbreviations for US states.}

\item{file_path}{The path to the directory where files are stored
(default is "inst/extdata/extracted_files").}

\item{verbose}{The stops the messages from printing to the console.}
}
\value{
A named list where each element is a list of data frames representing
confusion matrices for each state.
}
\description{
Function to retrieve confusion matrix data for multiple states of interest
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patch_sim.R
\name{generate_transition_vectors}
\alias{generate_transition_vectors}
\title{Function to generate transition vectors for negative cell values}
\usage{
generate_transition_vectors(r, transition_matrix, iterations = 10)
}
\arguments{
\item{r}{The SpatRaster object from get_patches with one layer for each
class.}

\item{transition_matrix}{The transition matrix you want to use.}

\item{iterations}{The number of iterations desired.}
}
\value{
A matrix of vector for each patch in each layer.
}
\description{
Function to generate transition vectors for negative cell values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_conf_mat.R
\name{get_trans_mat_dep}
\alias{get_trans_mat_dep}
\title{Function to format the confusion matrices as transition matrices}
\usage{
get_trans_mat_dep(df_list, categories)
}
\arguments{
\item{df_list}{A list of lists of data frames extracted using get_mat_data(). Each sublist should contain two or more data frames to be summed.}

\item{categories}{A list of categories defining the numbers between 1 and 256.}
}
\value{
A list of data frames where row names represent pixels that will
transition and column names represent the class they will transition to.
}
\description{
Function to format the confusion matrices as transition matrices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_conf_mat.R
\name{download_cdl_mat_files}
\alias{download_cdl_mat_files}
\title{Function to Read in and upzip confusion matrix files from the web}
\usage{
download_cdl_mat_files(years, temp_dir = "extracted_files")
}
\arguments{
\item{years}{The years of data that you want to be downloaded.}

\item{temp_dir}{The file name you want the extracted files to be stored in.}
}
\value{
The raw excel books containing the confusion matrix data for
every US state's USDA Crop Land Data Layer.
}
\description{
Function to Read in and upzip confusion matrix files from the web
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_conf_mat.R
\name{get_trans_mat}
\alias{get_trans_mat}
\title{Function to format the confusion matrices as transition matrices}
\usage{
get_trans_mat(df_list, categories)
}
\arguments{
\item{df_list}{A single data frame or a list of lists of data frames extracted using get_mat_data().
If a list, each sublist should contain two or more data frames to be summed.}

\item{categories}{A list of categories defining the numbers between 1 and 256.}
}
\value{
A list of data frames where row names represent pixels that will
transition and column names represent the class they will transition to.
}
\description{
Function to format the confusion matrices as transition matrices
}
\examples{
# make the data frame
bl_mat_data <- get_mat_data(c("UT"))

# List of categories with their corresponding vectors
# Define the values that represent our classes of interest
non_ag <- c(61:65, 81:83, 87:88, 92, 111:112, 121:124, 131, 141:143, 152, 176, 181, 190, 195)
alfalfa <- c(36:37)
major_ag <- c(1, 2, 5, 12, 13, 22:24, 26, 225:226, 228, 234, 236, 238:241, 254)
all_numbers <- 1:256
ag <- setdiff(all_numbers, c(non_ag, alfalfa, major_ag))

cat_5 <- list(non_ag = non_ag, ag = ag, alfalfa = alfalfa, major_ag = major_ag)

# get the confusion matrix for just 2008
trans_mat_5 <- get_trans_mat(bl_mat_data, cat_5)
}
